<?php
/**
 * Viper2 Framework
 *
 * @author fish <fish@archa.cz>
 * @package Viper2
 * @version 0.1
 *
 */
/**
 *  plugin pro spravu profilu - registrace/editace profilu
 *
 * @package Profiles
 * @author fish <fish@archa.cz>
 * @version 0.1
 *
 */

require_once INC_DIR . "/sendmail.php";
require_once LIBS_DIR . '/composer/archa/fred/flogin.php';
require_once LIBS_DIR . '/composer/archa/fred/flogin_user.php';

class Profiles extends Viper2Plugin {
  /**
   * variable $cfgProfileNew
   * @var array pole s definici formulare pro vytvoreni profilu
   */
  protected $cfgProfileNew = NULL;
  /**
   * variable $cfgProfileEdit
   * @var array pole s definici formulare pro editaci profilu
   */
  protected $cfgProfileEdit = NULL;
  /**
   * variable $cfgMailFrom
   * @var string odchozi adresa registracnich mailu
   */
  protected $cfgMailFrom = NULL;
  /**
   * variable $cfgMailAdmin
   * @var string e-mail administratora webu
   */
  protected $cfgMailAdmin = NULL;
  /**
   * variable $cfgMailSubject
   * @var string predmet mail s informacema o registraci
   */
  protected $cfgMailSubject = "Registrace";
  /**
   * variable $cfgVerifyReg
   * @var string ma se overit uzivatel poslanim hashe na jeho mail pri registraci ?
   */
  protected $cfgVerifyReg = TRUE;
  /**
   * variable $cfgSendInfo
   * @var string maji se poslat prihlasovaci udaje uzivateli pri registraci ?
   */
  protected $cfgSendInfo = TRUE;
  /**
   * variable $cfgMailSrc
   * @var string zdrojak mailu
   */
  protected $cfgMailSrc = NULL;
  /**
   * variable $cfgLoginDb
   * @var string schema s login tabulkama
   */
  protected $cfgLoginDb = NULL;
  /**
   * variable $cfgRegActions
   * @var array akce, ktery se uzivateli povolej pri registraci
   */
  protected $cfgRegActions = array();
  /**
   * variable $cfgConfActions
   * @var array akce, ktery se uzivateli povolej po potvrzeni registrace
   */
  protected $cfgConfActions = array();
  /**
   * variable $cfgXMLSync
   * @var boolean volat synchronizaci pres XML ?
   */
  protected $cfgXMLSync = FALSE;
  /**
   * variable $cfgUseSha1
   * @var boolean boolean heslo je ulozene jako SHA1
   */
  protected $cfgUseSha1 = FALSE;
  
  function __construct() {
    $this->pluginName = "Profiles";
    $this->tplPath = "Profiles/";
    $this->pluginUri = "/profil/";
    $this->tblPrefix = "";
    $this->cssLink = "/css/profiles/profiles.css";
        
  }
  
  public function actionGlobal() {
    if (!isset($this->cfgMailSrc)){
      throw new Exception("Není nastavená template registračního mailu");
    }
    if (!isset($this->cfgMailFrom)){
      throw new Exception("Není nastavený mail odesílatele");
    }
  }

  // {{{ actionRegistrace
  public function actionRegistrace($params) {
    if (isset($_POST["profiles_saveit"]) && $_POST["profiles_saveit"]=="yes") {
      if (!ValidateForm($this->cfgProfileNew, $_POST)) {
        v2("Msgbox", "add", _("Registrační formulář není správně vyplněn. Opravte prosím zadané údaje."));
      } else {
        //registrace
        $user = new flogin_user($this->dbh);
        $user->useSha1 = $this->cfgUseSha1;
        if (isset($this->cfgLoginDb)){
          $user->login_db="`".$this->cfgLoginDb."`.";
        }
        foreach ($this->cfgProfileNew as $valname=>$info) {
          if ($info['type']=='label') {
            continue;
          }
          if ($info["type"]=="copy" && isset($info["default"]) && isset($_POST[$info["default"]])) {
            $_POST[$valname] = $_POST[$info["default"]];
          }
          if (preg_match("/^__[^_]/", $valname)) {
            // jenom testovaci hodnota(souhlas s pravidlama atp.)
          } else if (preg_match("/^_[^_]/", $valname)) {
            if (isset($_POST[$valname])) {
              $user->val[$valname] = $_POST[$valname];
            }
          } else {
            if (isset($_POST[$valname])) {
              $user->$valname = $_POST[$valname];
            }
          }
        }
        $user->name = $user->fname." ".$user->lname;
        //pre_print($user);
        if ($user->save()) {
          if (isset($this->cfgVerifyReg) && $this->cfgVerifyReg===TRUE) {
            $reghash = md5($user->login.substr(rand(100000,999999),-5));
            flogin_user::setUserVal($this->dbh, $user->id_user, "_REG_HASH", $reghash);
            flogin_user::setUserVal($this->dbh, $user->id_user, "_REG_DATE", time());
            $this->sendActivationLink($reghash, $user);
            v2("Msgbox","sendOk",_("Registrace byla zahájena. Na Váš e-mail byl zaslán potvrzující kód pro dokončení registrace. Pokračujte dle instrukcí v zaslaném mailu."));
            Viper2::$router->redirectTo("Default::default");
          } else {
            if (isset($this->cfgSendInfo) && $this->cfgSendInfo===TRUE) {
              if (preg_match("/\.tpl$/", $this->cfgMailSrc)) {
                Viper2::$smarty->assign("REG_HASH", $reghash);
                if (isset($this->cfgSendInfo) && $this->cfgSendInfo===TRUE) {
                  Viper2::$smarty->assign("LOGIN", $this->login);
                  Viper2::$smarty->assign("PWD", $this->pwd);
                }
                $mailsrc = Viper2::$smarty->fetch($this->cfgMailSrc);
              } else {
                //txt mail source
                $mailsrc ="";
                $mailsrc.=_("Vaše registrace byla dokončena. Nyní se můžete přihlásit")."\n";
                $mailsrc.="\n";
                $mailsrc.=_("Váš login je").": ".$user->login."\n";
                $mailsrc.=_("Vaše heslo je").": ".$user->pwd."\n";
              }
              sendmail($this->cfgMailFrom, $user->mail, $this->cfgMailSubject, $mailsrc);
            }
            
            if ($this->cfgXMLSync) {
              file_get_contents(Viper2::$router->basePage()."/sys/xmlsync/user/".$user->id_user."/new.htm");
            }
            
            v2("Msgbox","sendOk", _("Registrace provedena, nyní se múžete přihlásit."));
            Viper2::$router->redirectTo("Default::default");
          }
        } else {
          v2("Msgbox","add", $user->status);
        }
      }
    }
  }
  // }}} actionRegistrace
  
  // {{{ renderRegistrace
  public function renderRegistrace($params, &$s) {
    $s->assign("fForm", $this->cfgProfileNew);
    $s->display($this->getTplPath()."new.tpl");
  }
  // }}} renderRegistrace

  // {{{ actionEditace
  public function actionEditace($params) {
    if (Viper2::getVar("logged")===TRUE && Viper2::getVar("loginUid")!==NULL) {
      $user = new flogin_user($this->dbh, Viper2::getVar("loginUid"));
      $user->useSha1 = $this->cfgUseSha1;
      if (isset($_POST["profiles_saveit"]) && $_POST["profiles_saveit"]=="yes") {
        if (!ValidateForm($this->cfgProfileEdit, $_POST)) {
          v2("Msgbox", "add", _("Registrační formulář není správně vyplněn. Opravte prosím zadané údaje."));
        } else {
          //editace
          foreach ($this->cfgProfileEdit as $valname=>$info) {
            if ($info['type']=='label') {
              continue;
            }
            if ($info["type"]=="copy" && isset($info["default"]) && isset($_POST[$info["default"]])) {
              $_POST[$valname]=$_POST[$info["default"]];
            }
            if (preg_match("/^__[^_]/", $valname)) {
              // jenom testovaci hodnota(souhlas s pravidlama atp.)
            } else if (preg_match("/^_[^_]/", $valname)) {
              if (isset($_POST[$valname]) && empty($info["static"])) {
                $user->val[$valname] = $_POST[$valname];
              }
            } else {
              if (isset($_POST[$valname]) && empty($info["static"])) {
                $user->$valname=$_POST[$valname];
              }
            }
          }
          $user->name = $user->fname." ".$user->lname;
          if ($res=$user->save()) {
            if ($this->cfgXMLSync) {
              if (isset($user->val["_ODBYT"]) && $user->val["_ODBYT"]*1!=0) {
                file_get_contents(_BASE_PAGE."/sys/xmlsync/user/".$user->id_user."/mod.htm");
              } else {
                file_get_contents(_BASE_PAGE."/sys/xmlsync/user/".$user->id_user."/new.htm");
              }
            }
            v2("Msgbox", "addOk", _("Změna provedena"));
          } else {
            v2("Msgbox", "add", $res->getMessage());
          }
        }
      } else {
        if (!isset($_POST["profiles_saveit"]) || $_POST["profiles_saveit"]=="reload") {
          // pre_print($this->cfgProfileEdit);
          foreach ($this->cfgProfileEdit as $valname=>$info) {
            // echo $valname._nl;
            if (preg_match("/^__[^_]/", $valname)) {
              // jenom testovaci hodnota(souhlas s pravidlama atp.)
            } else if (preg_match("/^_[^_]/", $valname)) {
              if (isset($user->val[$valname]) && ((isset($_POST["profiles_saveit"]) && empty($info["static"]) && $_POST["profiles_saveit"]!="reload") || empty($_POST[$valname]))) {
                $_POST[$valname] = $user->val[$valname];
              }
            } else {
              if (isset($user->$valname) && ((isset($_POST["profiles_saveit"]) && empty($info["static"]) && $_POST["profiles_saveit"]!="reload") || empty($_POST[$valname]))) {
                $_POST[$valname] = $user->$valname;
              }
            }
          }
          // pre_print($_POST);
        }
      }
    } else {
      v2("Msgbox", "add", _("Pro editaci profilu se nejprve musíte přihlásit"));
      Viper2::$router->transferTo("::_restricted");
    }
  }
  // }}} actionEditace

  // {{{ renderEditace
  public function renderEditace($params, &$s) {
    $s->assign("fForm", $this->cfgProfileEdit);
    $s->display($this->getTplPath()."edit.tpl");
  }
  // }}} renderEditace

  // {{{ actionPotvrzeni
  public function actionPotvrzeni($params) {
    $reg_hash = preg_replace("/[^a-z0-9]/i","",isset($_GET["hash"])?$_GET["hash"]:"");
    $query ="SELECT user_classes.id_user FROM (";
    if (isset($this->cfgLoginDb)) {
      $query.="`".$this->cfgLoginDb."`.";
    }
    $query.="user_classes,";
    if (isset($this->cfgLoginDb)) {
      $query.="`".$this->cfgLoginDb."`.";
    }
    $query.="classes) WHERE classes.varname=".$this->dbh->quote("REG_HASH")." AND user_classes.id_class=classes.id ";
    $query.="AND user_classes.val=".$this->dbh->quote($reg_hash)." ";
//        echo $query._nl;
    $res = $this->dbh->query($query);
    if (!PEAR::isError($res)) {
      if ($row=$res->fetchRow()) {
        flogin_user::setUserVal($this->dbh, $row["id_user"], "_REG_HASH", "", $this->cfgLoginDb);
        if (count($this->cfgConfActions)) {
          foreach ($this->cfgConfActions as $action) {
            flogin::setUserAct($this->dbh, $row["id_user"], $action["sys"], $action["act"], TRUE, $this->cfgLoginDb);
          }
        }
        if ($this->cfgXMLSync) {
          file_get_contents(Viper2::$router->basePage()."/sys/xmlsync/user/".$row["id_user"]."/new.htm");
        }
        v2("Msgbox", "sendOk", _("Vaše registrace byla dokončena. Nyní se můžete přihlásit."));
        Viper2::$router->redirectTo("Default::default");
      } else {
        v2("Msgbox", "addOk", _("Neplatný registrační kód !!!")._nl._("(Buď jste špatně otevřeli odkaz zaslaný e-mailem, nebo se pokoušíte otevřít odkaz, který již byl použit pro dokončení registrace)"));
      }
    } else {
      v2("Msgbox", "addOk", $res->getMessage()."; ".$res->getUserInfo());
    }
  }
  // }}} actionPotvrzeni

  // {{{ actionHeslo
  public function actionHeslo($params) {
    if (isset($_REQUEST["send_password"]) && $_REQUEST["send_password"]=="yes") {
      $Email=!empty($_REQUEST["reset"])?$_REQUEST["reset"]:"";
  
      $query ="SELECT id_user FROM users ";
      $query.="WHERE mail LIKE ".$this->dbh->quote($Email)." ";
      if (!PEAR::isError($res=$this->dbh->getAll($query))) {
        if (count($res)>0) {
          $RandomPwd = self::createRandomPassword(8);
          $insertOk = TRUE;
          foreach ($res as $userInfo) {
            $query ="INSERT INTO users_otp(id_user,otp,expire) ";
            $query.="VALUES(".$this->dbh->quote($userInfo["id_user"]*1).",".$this->dbh->quote($this->cfgUseSha1?sha1($RandomPwd):md5($RandomPwd)).",ADDDATE(NOW(),INTERVAL 1 HOUR)) ";
            if (PEAR::isError($this->dbh->query($query))) {
              $userInfo = FALSE;
              break;
            }
          }
          if ($insertOk) {
            $mailsrc = Viper2::$smarty->fetch($this->getTplPath()."password_mail.tpl");
            $mailsrc = preg_replace("/#PASSWORD#/", $RandomPwd, $mailsrc);
            $mailsrc_html = nl2br($mailsrc);
            sendmail2($this->cfgMailFrom, $Email, _("Zapomenuté heslo"), $mailsrc, $mailsrc_html);
            v2("Msgbox", "sendOk", "Na adresu <b>".htmlspecialchars($Email)."</b> byl odeslán e-mail s instrukcemi pro změnu hesla.");
            Viper2::$router->reloadPage();
          } else {
            v2("Msgbox", "add", _("Při odesílání hesla došlo k chybě"));
          }
        } else {
          v2("Msgbox", "add", _("Pro zadanou e-mailovou adresu nebyl nalezen platný účet"));
        }
      }
    }
  }
  // }}} actionHeslo

  // {{{ renderHeslo
  public function renderHeslo($params, &$s) {
    $s->display($this->getTplPath()."password.tpl");
  }
  // }}} renderHeslo
  
  // {{{ createRandomPassword
  /**
   * vygeneruje nahodne heslo
   *
   * @return string heslo
   */
  static function createRandomPassword($MaxLen=8) {
    $chars = "ABCDEFGHIJKMNPQRSTUVWXabcdefghijkmnopqrstuvwx23456789";
    srand((double)microtime()*1000000);
    $i = 0;
    $pass = '' ;
    while ($i < $MaxLen) {
        $num = rand() % strlen($chars);
        $tmp = substr($chars, $num, 1);
        $pass.=$tmp;
        $i++;
    }
    return $pass;
  }
  // }}} createRandomPassword
  
  function sendActivationLink($reghash, $user) {
    if (preg_match("/\.tpl$/", $this->cfgMailSrc)) {
      Viper2::$smarty->assign("REG_HASH", $reghash);
      if (isset($this->cfgSendInfo) && $this->cfgSendInfo===TRUE) {
        Viper2::$smarty->assign("LOGIN", $user->login);
        Viper2::$smarty->assign("PWD", $user->pwd);
      }
      $mailsrc = Viper2::$smarty->fetch($this->cfgMailSrc);
    } else {
      //txt mail source
      $mailsrc ="";
      $mailsrc.=_("Pro potvrzeni registrace otevrete tento odkaz:\n");
      $mailsrc.=_("Nyní je třeba potvrdit Vaši registraci.")."\n\n";
      $mailsrc.=_("Jděte na stránku:")."\n";

      $mailsrc.=Viper2::$router->basePage().Viper2::$router->makeLink(array("to"=>"Profiles::potvrzeni"))."?hash=".$reghash."\n\n";

      $mailsrc.=_("Klikněte na výše uvedenou adresu.")."\n\n";
      $mailsrc.=_("Upozornění! Někteří uživatelé freemailových služeb (např. Seznam.cz, centrum.cz, ...) mohou mít problém s funkčností odkazu. V takovém případě adresu jednoduše zkopírujte (označit a stisknout kombinaci Ctrl+C)")."\n";

      if (isset($this->cfgSendInfo) && $this->cfgSendInfo===TRUE) {
        $mailsrc.="\n";
        $mailsrc.=_("Váš login je:")." ".$user->login."\n";
        if (!empty($user->pwd)) {
          $mailsrc.=_("Vaše heslo je:")." ".$user->pwd."\n";
        }
      }
    }
    sendmail($this->cfgMailFrom, $user->mail, $this->cfgMailSubject, $mailsrc);
  }
  
  function renderResend($params, &$s) {
    $regHash = preg_replace("/[^a-z0-9]/i","",isset($_GET["hash"])?$_GET["hash"]:"");
    
    if (!empty($regHash)) {
      $query ="SELECT user_classes.id_user FROM (";
      if (isset($this->cfgLoginDb)) {
        $query.="`".$this->cfgLoginDb."`.";
      }
      $query.="user_classes,";
      if (isset($this->cfgLoginDb)) {
        $query.="`".$this->cfgLoginDb."`.";
      }
      $query.="classes) WHERE classes.varname=".$this->dbh->quote("REG_HASH")." AND user_classes.id_class=classes.id ";
      $query.="AND user_classes.val=".$this->dbh->quote($regHash)." ";
      $query.= "LIMIT 0,1 ";
      $res = $this->dbh->query($query);
      if (!PEAR::isError($res)) {
        if ($row=$res->fetchRow()) {
          $user = new flogin_user($this->dbh);
          $user->useSha1 = $this->cfgUseSha1;
          $user->load($row["id_user"]);
          $this->sendActivationLink($regHash, $user);          
          v2("Msgbox", "put", _("Potvrzovací e-mail odeslán"));
        } else {
          v2("Msgbox", "put", _("Neplatný registrační kód !!!"));
        }
      } else {
        v2("Msgbox", "put", $res->getMessage()."; ".$res->getUserInfo());
      }
    }

  }

}
